package company_product_try_out_update_log

import (
	"github.com/beego/beego/v2/client/orm"
	"hongze/hongze_mobile_admin/models/tables/company_product_try_out_update_permission_log"
	"hongze/hongze_mobile_admin/models/tables/company_report_permission"
	"time"
)

// CompanyProductTryOutUpdateLog 客户产品试用变更日志表
type CompanyProductTryOutUpdateLog struct {
	Id          int       `orm:"column(id);pk"`
	CompanyId   int       `description:"客户id"`
	ProductId   int       `description:"产品id"`
	SellerId    int       `description:"销售id"`
	SellerName  string    `description:"销售名称"`
	Source      string    `description:"来源"`
	StartDate   time.Time `description:"开始日期"`
	EndDate     time.Time `description:"结束日期"`
	RealEndDate time.Time `description:"实际结束日期"`
	IsStop      int       `description:"是否已经终止了，0：进行中；1：已终止"`
	CreateTime  time.Time `description:"创建时间"`
}

// Update 更新客户产品试用变更日志
func (item *CompanyProductTryOutUpdateLog) Update(cols []string) (err error) {
	o := orm.NewOrm()
	_, err = o.Update(item, cols...)
	return
}

// AddCompanyProductTryOutUpdateLog 新增客户试用产品变更日志
func AddCompanyProductTryOutUpdateLog(item *CompanyProductTryOutUpdateLog, companyReportPermissionList []*company_report_permission.CompanyReportPermission) (err error) {
	o := orm.NewOrm()
	lastId, err := o.Insert(item)
	if err != nil {
		return
	}
	item.Id = int(lastId)

	companyProductUpdatePermissionLogList := make([]*company_product_try_out_update_permission_log.CompanyProductTryOutUpdatePermissionLog, 0)
	for _, v := range companyReportPermissionList {
		tmpCompanyProductUpdatePermissionLog := &company_product_try_out_update_permission_log.CompanyProductTryOutUpdatePermissionLog{
			//Id:                        0,
			CompanyProductUpdateLogId: item.Id,
			ChartPermissionId:         v.ChartPermissionId,
			StartDate:                 item.StartDate,
			EndDate:                   item.EndDate,
			RealEndDate:               item.StartDate,
			IsStop:                    item.IsStop,
			CreateTime:                item.CreateTime,
		}
		companyProductUpdatePermissionLogList = append(companyProductUpdatePermissionLogList, tmpCompanyProductUpdatePermissionLog)
	}

	if len(companyProductUpdatePermissionLogList) > 0 {
		_, err = o.InsertMulti(len(companyProductUpdatePermissionLogList), companyProductUpdatePermissionLogList)
	}

	{
		// 获取当前正在进行中的客户变更试用的数据
		findProgress, _ := GetCompanyProductTryOutUpdateLogItem(item.CompanyId, item.ProductId)
		// 如果没有的话，那么就将试用天数+1
		if findProgress != nil {
			sql := `UPDATE  company_product
			SET
			  try_out_day_total = try_out_day_total+1
			WHERE company_id = ? AND product_id = ?`
			_, err = o.Raw(sql, item.CompanyId, item.ProductId).Exec()
		}
	}
	return
}

// GetCompanyProductTryOutUpdateLogItem 获取当前正在进行中的客户变更试用的数据
func GetCompanyProductTryOutUpdateLogItem(companyId, productId int) (item *CompanyProductTryOutUpdateLog, err error) {
	o := orm.NewOrm()
	sql := ` select * from company_product_try_out_update_log where company_id=? and product_id=? and is_stop = 0`
	err = o.Raw(sql, companyId, productId).QueryRow(&item)
	return
}
