package company_remind_record

import (
	"github.com/beego/beego/v2/client/orm"
	"time"
)

// CompanyRemindRecord 客户到期提醒记录
type CompanyRemindRecord struct {
	CompanyRemindRecordId int    `orm:"column(company_remind_record_id);pk"`
	Type                  int    `description:"过期类型，1:1天，2:7天，3:15天；4:30天"`
	SellerId              int    `description:"销售id"`
	SellerName            string `description:"销售名称"`
	CompanyId             int    `description:"客户id"`
	CompanyName           string `description:"客户名称"`
	Status                string `description:"客户状态"`
	EndDate               string `description:"到期日期"`
	UniqueCode            string `description:"唯一code"`
	CreateTime            time.Time
}

// GetAllCompanyRemindRecordList 获取所有的客户到期提醒记录
func GetAllCompanyRemindRecordList(uniqueCode string) (list []*CompanyRemindRecord, err error) {
	o := orm.NewOrm()
	sql := `SELECT * FROM company_remind_record WHERE unique_code=? `

	_, err = o.Raw(sql, uniqueCode).QueryRows(&list)
	return
}

// CompanyRemindRecordResp 客户即将过期列表返回
type CompanyRemindRecordResp struct {
	RemindType int                    `description:"过期类型，1:1天，2:7天，3:15天；4:30天"`
	Total      int                    `description:"总数量"`
	EndDate    string                 `description:"到期日期"`
	List       []*CompanyRemindRecord `description:"数据列表"`
}
