package company_report_permission

import (
	"github.com/beego/beego/v2/client/orm"
	"time"
)

type CompanyReportPermission struct {
	CompanyReportPermissionId int64 `orm:"column(company_report_permission_id);pk"`
	CompanyId                 int
	ReportPermissionId        int
	CreatedTime               time.Time
	LastUpdatedTime           time.Time
	ChartPermissionId         int
	StartDate                 string    `description:"权限开始日期"`
	EndDate                   string    `description:"权限结束日期"`
	ProductId                 int       `description:"产品id"`
	ProductName               string    `description:"产品名称"`
	CompanyContractId         int       `description:"合同id"`
	Status                    string    `description:"'正式','试用','关闭'"`
	ModifyTime                time.Time `description:"修改时间"`
	IsUpgrade                 int       `description:"是否升级，1是，0否"`
	ExpensiveYx               int       `description:"权益研选: 0-3w; 1-5w"`
}

func GetCompanyReportPermissionCount(companyId, productId int) (count int, err error) {
	o := orm.NewOrm()
	sql := `SELECT COUNT(1) AS count FROM company_report_permission WHERE company_id=? AND product_id=? `
	err = o.Raw(sql, companyId, productId).QueryRow(&count)
	return
}

func DeleteCompanyReportPermission(companyId, productId int) (err error) {
	o := orm.NewOrm()
	sql := ` DELETE FROM company_report_permission WHERE company_id=? AND product_id=? `
	_, err = o.Raw(sql, companyId, productId).Exec()
	return
}

type PermissionLookList struct {
	ClassifyName string `description:"分类"`
	Items        []*PermissionLookItem
	CheckList    []int
}

type PermissionLookItem struct {
	ChartPermissionId int     `description:"权限id"`
	PermissionName    string  `description:"权限名称"`
	StartDate         string  `description:"权限开始日期"`
	EndDate           string  `description:"权限结束日期"`
	Status            string  `description:"'正式','试用','关闭'"`
	ExpireDay         string  `description:"到期天数"`
	ClassifyName      string  `description:"分类"`
	Remark            string  `description:"备注"`
	IsUpgrade         int     `description:"是否升级，1是，0否"`
	ExpensiveYx       int     `description:"权益研选: 0-3w; 1-5w"`
	Points            float64 `description:"研选扣点包点数"`
	ParentId          int     `description:"父级权限id"`
	IsPublic          int     `description:"是否是公有权限1:公有权限，0私有权限"`
}

func GetCompanyPermissionCheck(companyId, permissionId int) (count int, err error) {
	o := orm.NewOrm()
	sql := `SELECT COUNT(1) AS count FROM company_report_permission AS a WHERE a.company_id=? AND a.chart_permission_id=? `
	err = o.Raw(sql, companyId, permissionId).QueryRow(&count)
	return
}

func GetCompanyPermissionCheckItem(companyId, permissionId int) (item *PermissionLookItem, err error) {
	o := orm.NewOrm()
	sql := `SELECT * FROM company_report_permission AS a WHERE a.company_id=? AND a.chart_permission_id=? `
	err = o.Raw(sql, companyId, permissionId).QueryRow(&item)
	return
}

func GetPermissionLookItems(productId int, classifyName string) (items []*PermissionLookItem, err error) {
	o := orm.NewOrm()
	sql := ` SELECT * FROM chart_permission WHERE enabled=1 AND parent_id > 0 AND product_id=? AND classify_name=? ORDER BY sort ASC `
	_, err = o.Raw(sql, productId, classifyName).QueryRows(&items)
	return
}

func GetPermissionLookByParentId(productId int, parentId int) (items []*PermissionLookItem, err error) {
	o := orm.NewOrm()
	sql := ` SELECT * FROM chart_permission WHERE enabled=1 AND product_id=? AND parent_id=? ORDER BY sort ASC `
	_, err = o.Raw(sql, productId, parentId).QueryRows(&items)
	return
}

func GetPermissionLookItemsByProductId(productId int) (items []*PermissionLookItem, err error) {
	o := orm.NewOrm()
	sql := ` SELECT * FROM chart_permission WHERE enabled=1 AND product_id=? ORDER BY sort ASC `
	_, err = o.Raw(sql, productId).QueryRows(&items)
	return
}

// GetCompanyContractPermissionByCompanyContractId 根据合同ID获取开通的权限
func GetCompanyContractPermissionByCompanyContractId(companyContractId int) (items []*PermissionLookItem, err error) {
	o := orm.NewOrm()
	sql := `SELECT * FROM company_contract_permission WHERE company_contract_id =? ORDER BY modify_time DESC `
	_, err = o.Raw(sql, companyContractId).QueryRows(&items)
	return
}

// GetCompanyPermission 根据客户id和品种id获取对应的数据
func GetCompanyPermission(companyId, permissionId int) (item *CompanyReportPermission, err error) {
	o := orm.NewOrm()
	sql := `SELECT * FROM company_report_permission AS a WHERE a.company_id=? AND a.chart_permission_id=? `
	err = o.Raw(sql, companyId, permissionId).QueryRow(&item)
	return
}
